﻿/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.
 
The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.
 
Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intel’s suppliers or licensors in any way.

*/

using System.IO;
using System.Linq;

namespace PhotoApp.DataServices
{
    /// <remarks>
    /// Implementation of IDataService interface. Implements methods to access and store image/s.
    /// </remarks>    
    public class DataService : IDataService
    {
        /// <summary>
        /// Defines path to "pictures" directory which contains all web-cam captured images by the application.
        /// </summary>
        string picturesDirectory;

        public string PicturesDirectory
        {
            get
            {
                return this.picturesDirectory;
            }
        }
        /// <summary>
        /// Default constructor to get current "pictures" directory path for the applicaion.
        /// </summary>
        public DataService()
        {
            //Creates directory path string to "pictures" directory by referring to the application(*.exe) directory.
            picturesDirectory = Directory.GetCurrentDirectory() + "\\..\\" + "pictures";

            //Check if "pictures" directiry exist or not. If "pictures" directory does not exist then lets create it.
            if (!Directory.Exists(picturesDirectory))
                Directory.CreateDirectory(picturesDirectory);
        }

        /// <summary>
        /// Function to store image as specified image-name.
        /// </summary>
        /// <param name="imageBytes">Image bytes.</param>
        /// <param name="imageName">Image name used to store image.</param>
        public void StoreImage(byte[] imageBytes, string imageName)
        {
            //Get pictures directory path and combine it with image name.
            string imageFilePath = picturesDirectory + "\\" + imageName;

            //Create writer to write image bytes @ specified image file path.
            using (BinaryWriter bw = new BinaryWriter(File.Create(imageFilePath, imageBytes.Length)))
            {
                //Write bytes.
                bw.Write(imageBytes);
                //Close writer once done.
                //bw.Close();
            }
        }

        /// <summary>
        /// Function to get paths for all stored image files into "pictures" directory with specified file extension.
        /// </summary>
        /// <param name="fileExt">This is default parameter. Specify only if require to search for specific file-types. By default it searchs for all file types.</param>
        /// <returns> String array of file paths within the pictures folder.</returns>
        public string[] GetPathsOfAllStoredImages(string fileExt = "")
        {
            DirectoryInfo di = new DirectoryInfo(picturesDirectory);
            FileSystemInfo[] files;
            string[] filePaths = null;

            if (fileExt.Length > 0)
            {
                string searchPattern = string.Format("*.{0}", fileExt);
                //return Directory.GetFiles(picturesDirectory, searchPattern);

                //Get Files from directory, search by extensions.
                files = di.GetFileSystemInfos(searchPattern);
            }
            else
            {
                //return Directory.GetFiles(picturesDirectory);

                //Get files from directory.
                files = di.GetFileSystemInfos();
            }

            //Sort the files by creation date.
            var orderedFiles = files.OrderByDescending(f => f.CreationTime);

            if (orderedFiles != null)
            {
                filePaths = new string[orderedFiles.Count()];

                for (int i = 0; i < orderedFiles.Count(); i++)
                    filePaths[i] = orderedFiles.ElementAt(i).FullName;
            }

            return filePaths;
        }
    }
}
