﻿/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.
 
The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.
 
Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intel’s suppliers or licensors in any way.

*/

using System.Windows;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Controls;
using PhotoApp.ViewModel;
using System;
using System.Timers;
using System.ComponentModel;
using Logger;
using PhotoApp.Utility;
using Windows.Media.Capture;
using System.Windows.Interop;
using System.Windows.Controls.Primitives;
using System.IO;
using System.Windows.Input;
using PhotoApp.Controls;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.Threading;

namespace PhotoApp
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        //ViewModel of MainWindow
        MainWindowViewModel vm;
        //Contains tectual debug panel
        ScrollViewer imageScrollViewer;
        //Provides power management functions.
        PowerMngrHelper powerHelper;
        //CumulativeDeltaX to detect swipe
        private static double cumulativeDeltaX;
        //CumulativeDeltaY to detect swipe
        private static double cumulativeDeltaY;
        //Check if the element was actullay manipulated
        private static bool isManipulated = false;
        //Check if we need to StartWebCam or RestartWebcam
        private static bool shouldStartWebcam = true;
        //Linear velocity
        private static double linearVelocity;
        //To check if fullscreen mode or window mode
        private bool isFullscreenMode;
        //Actual Height of list box, used to set while orientation changed.
        double lbActualHeight;
        //Image's current X zoom scale.
        double imgCurrentScaleX = 1.0;
        //Image's current Y zoom scale.
        double imgCurrentScaleY = 1.0;
        //Flag to check wether camera capture enabled or disabled.
        bool cameraCaptureEnabled;
        //Flag to check wether location access enabled or disabled.
        bool locationAccessEnabled;
        //Application Name
        string applicationName;
        //Camera access permission
        bool canAccessCamera;
        //Location access permission
        bool canAccessLocation;
        //Battery below threshold.
        bool isBatteryBelowThreshold;
        //Camera accepted by user
        bool cameraAccepted;
        //Location accepted by user
        bool locationAccepted;
        //Permission window to access camera.
        MessageWindow accessCameraWindow;
        //Permission window to access location.
        MessageWindow accessLocationWindow;

        /// <summary>
        /// MainWindow constructor creates ViewModel instance and assigns to window's data context.
        /// Registers main window lifecycle events.
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            //Check log file size.
            CheckLogFileSize();

            //Initial set flag.
            cameraCaptureEnabled = true;
            locationAccessEnabled = true;

            vm = new MainWindowViewModel();
            this.DataContext = vm;

            //Register to get orientation change notification.
            vm.OrientationChanged += vm_OrientationChanged;
            //Register to get device shaked notification.
            vm.DeviceShaked += vm_DeviceShake;
            //Register to get battery below threshold notification.
            vm.BatteryBelowThreshold += vm_BatteryBelowThreshold;
            //Register to get battery above threshold notification.
            vm.BatteryAboveThreshold += vm_BatteryAboveThreshold;

            this.Loaded += MainWindow_Loaded;
            this.Closing += MainWindow_Closing;

            //Load images.
            vm.LoadImages();
        }

        //Notification method invoked when battery is above threshold value or power is plugged in.
        void vm_BatteryAboveThreshold(object sender, EventArgs e)
        {
            isBatteryBelowThreshold = false;

            //Enable camera capturing
            if (!cameraCaptureEnabled && canAccessCamera)
            {
                if (PowerMngrHelper.GetPowerState().LineStatus == ACLineStatus.Online)
                {
                    //ToastMsgHelper.ShowToastMessage("Power is plugged in, camera capture has been enabled");
                    EnableCamera(true);
                }
                else if (!cameraAccepted)
                {
                    //Ask user to enable.
                    if (AskCameraPermission("Battery is above threshold value, do you want to enable camera capture?"))
                    {
                        cameraAccepted = true;
                        //ToastMsgHelper.ShowToastMessage(AppConstants.EnableCamera);
                        EnableCamera(true);
                    }
                }
            }

            //Enable location access
            if (!locationAccessEnabled && canAccessLocation)
            {
                if (PowerMngrHelper.GetPowerState().LineStatus == ACLineStatus.Online)
                {
                    //Automatic enable geo location if power plugged in. Only if user has not clicked to set manually.
                    if (!vm.GeoLocationEnable && !vm.userClickedGeoTagBtn)
                    {
                        vm.GeoLocationEnable = true;
                        //ToastMsgHelper.ShowToastMessage("Power is plugged in, location access has been enabled");
                    }

                    EnableLocation(true);
                }
                else
                {
                    //Ask to enable geo location if battery percentage above Threshold value. Only if user has not click to set manually.
                    if (!vm.GeoLocationEnable && !vm.userClickedGeoTagBtn && !locationAccepted)
                    {
                        //Ask user to enable.
                        if (AskLocationPermission("Battery is above threshold value, do you want to enable location access?"))
                        {
                            locationAccepted = true;
                            //ToastMsgHelper.ShowToastMessage(AppConstants.EnableLocation);
                            EnableLocation(true);

                            vm.GeoLocationEnable = true;
                        }
                    }
                }
            }
        }

        //Notification method invoked when battery is below threshold value.
        void vm_BatteryBelowThreshold(object sender, EventArgs e)
        {
            isBatteryBelowThreshold = true;

            //Disable camera capturing
            if (cameraCaptureEnabled && canAccessCamera && !cameraAccepted)
            {
                //ToastMsgHelper.ShowToastMessage(AppConstants.DisableCamera);
                EnableCamera(false);
            }

            //Disable location access
            if (locationAccessEnabled && canAccessLocation && !locationAccepted)
            {
                //Automatic disable geo location if battery percentage falls down below Threshold value. Only if user has not click to set manually.
                if (vm.GeoLocationEnable && !vm.userClickedGeoTagBtn)
                {
                    vm.GeoLocationEnable = false;
                    //ToastMsgHelper.ShowToastMessage(AppConstants.DisableLocation);
                }

                EnableLocation(false);
            }
        }

        private bool AskCameraPermission(string permission)
        {
            double screenHeight = SystemParameters.PrimaryScreenHeight;
            double screenWidth = SystemParameters.PrimaryScreenWidth;

            //Ask for permission.
            MessageWindow accessCamera = new MessageWindow();
            accessCamera.allowBtn.Content = "Yes";
            accessCamera.cancelBtn.Content = "No";
            accessCamera.Width = screenWidth;
            accessCamera.Top = screenHeight / 2;
            accessCamera.Left = 0;
            accessCamera.Message = permission;
            this.Opacity = 0.5;
            accessCamera.ShowDialog();
            this.Opacity = 1.0;

            return accessCamera.isAllowed;
        }

        private bool AskLocationPermission(string permission)
        {
            double screenHeight = SystemParameters.PrimaryScreenHeight;
            double screenWidth = SystemParameters.PrimaryScreenWidth;

            //Ask for permission.
            MessageWindow accessLocation = new MessageWindow();
            accessLocation.allowBtn.Content = "Yes";
            accessLocation.cancelBtn.Content = "No";
            accessLocation.Width = screenWidth;
            accessLocation.Top = screenHeight / 2;
            accessLocation.Left = 0;
            accessLocation.Message = permission;
            this.Opacity = 0.5;
            accessLocation.ShowDialog();
            this.Opacity = 1.0;

            return accessLocation.isAllowed;
        }

        private void CheckLogFileSize()
        {
            //Check the log file size and if it is above 5 MB then ask user to delete it.
            try
            {
                FileInfo f = new FileInfo(AppConstants.AppLogFileName);
                if (f != null)
                {
                    long s1 = f.Length;
                    if (s1 >= 5 * 1024 * 1024)  //5 MB
                    {
                        if (MessageBox.Show("Do you want to delete it?", "Log file size is above 5 MB", MessageBoxButton.YesNo) == MessageBoxResult.Yes)
                        {
                            File.Delete(AppConstants.AppLogFileName);
                        }
                    }
                }
            }
            catch (Exception em)
            {
                AppLogger.LogToFile(em.Message);
            }
        }

        //Notification method called when device shaked.
        void vm_DeviceShake(object sender, EventArgs e)
        {
            //Device shaken so show the random image from list.
            Dispatcher.BeginInvoke(new Action(() => ShowRandomImage()), null);
        }

        //Shows random image from images list.
        void ShowRandomImage()
        {
            //No need to show the random image, if camera is running.
            if (cameraBtn.Visibility == Visibility.Visible)
                return;

            int count = 0;
            if (vm.ImageList != null)
                count = vm.ImageList.Count;

            if (count > 0)
            {
                Random rand = new Random();
                int num = rand.Next(0, count - 1);

                imagesListBox.SelectedIndex = num;
            }

        }

        /// <summary>
        /// Called when the orientation changed
        /// </summary>
        /// <param name="sender">ViewModel class is the sender of the event</param>
        /// <param name="e">Orientation Changed argument to get the current orientation</param>
        void vm_OrientationChanged(object sender, OrientationChangedEventArgs e)
        {
            //Change the view layout.
            Thread.Sleep(500);

            Dispatcher.BeginInvoke(new Action(() => ChangeLayout(e.viewOrientation)), null);
        }

        /// <summary>
        /// Change View layout as per the current orientation mode.
        /// </summary>
        /// <param name="currentOrienationMode">current orientaion mode of view.</param>
        void ChangeLayout(ViewOrientationMode currentOrienationMode)
        {

            //Portrait mode
            if (currentOrienationMode == ViewOrientationMode.OrientationModeFaceUp ||
                currentOrienationMode == ViewOrientationMode.OrientationModeFaceDown ||
                currentOrienationMode == ViewOrientationMode.OrientationModePortraitUp ||
                currentOrienationMode == ViewOrientationMode.OrientationModePortraitDown)
            {
                //Change the layouts for controls.
                Grid.SetRow(debugPanel, 0);
                Grid.SetColumn(debugPanel, 1);
                debugGraphicaPanel.Orientation = Orientation.Vertical;

                Grid.SetRow(listBoxGrid, 0);
                Grid.SetColumn(listBoxGrid, 0);
                Grid.SetColumnSpan(listBoxGrid, 1);

                Grid.SetRow(controlsGrid, 0);
                Grid.SetColumn(controlsGrid, 1);

                Grid.SetColumn(controlsBtnPanel, 0);
                Grid.SetColumn(cameraBtn, 1);

                imagesListBox.Height = lbActualHeight;

                //Reset the orientation for camera button & controls panel.
                controlsGrid.HorizontalAlignment = HorizontalAlignment.Right;
                cameraBtn.HorizontalAlignment = HorizontalAlignment.Right;

                //Change the scrollbar visibility.
                if (imageScrollViewer != null)
                    imageScrollViewer.HorizontalScrollBarVisibility = ScrollBarVisibility.Hidden;
                try
                {
                    /*
                    ROTATE_NONE						= 0,
                    ROTATE_90_DEGREES_CLOCKWISE		= 1, //issue
                    ROTATE_180_DEGREES_CLOCKWISE	= 2,
                    ROTATE_270_DEGREES_CLOCKWISE	= 3, //issue
                    ROTATE_FLIP_VERTICAL			= 4,
                    ROTATE_FLIP_HORIZONTAL			= 5,
                    ROTATE_FLIP_DIAGONALLY			= 6 //issue
                    */
                    if (currentOrienationMode == ViewOrientationMode.OrientationModeFaceUp || currentOrienationMode == ViewOrientationMode.OrientationModePortraitUp)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(0);
                        }
                    }
                    else if (currentOrienationMode == ViewOrientationMode.OrientationModeFaceDown || currentOrienationMode == ViewOrientationMode.OrientationModePortraitDown)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(2);
                        }
                    }
                }
                catch (DllNotFoundException Dexcp)
                {
                    MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
                }
                catch (FormatException fexcp)
                {
                    MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
                }
                catch (Exception excp)
                {
                    MessageBox.Show(excp.Message);
                }

                //Change the orientation image to portrait.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath +  "portrait.png", UriKind.RelativeOrAbsolute));
                if (bi != null) orientationImage.Source = bi;

            }
            else if (currentOrienationMode == ViewOrientationMode.OrientationModeLandScapeLeft ||
                currentOrienationMode == ViewOrientationMode.OrientationModeLandScapeRight) //LandScape mode
            {
                //Change the layouts for controls.
                Grid.SetRow(debugPanel, 1);
                Grid.SetColumn(debugPanel, 0);
                debugGraphicaPanel.Orientation = Orientation.Horizontal;

                Grid.SetRow(listBoxGrid, 1);
                Grid.SetColumn(listBoxGrid, 0);
                Grid.SetColumnSpan(listBoxGrid, 2);

                Grid.SetRow(controlsGrid, 0);
                Grid.SetColumn(controlsGrid, 0);

                //Set the image list box to show only 2 rows.
                imagesListBox.Height = 275;

                //Set the orientation for camera button & controls panel accordingly.
                if (currentOrienationMode == ViewOrientationMode.OrientationModeLandScapeRight)
                {
                    controlsGrid.HorizontalAlignment = HorizontalAlignment.Right;

                    Grid.SetColumn(controlsBtnPanel, 0);
                    Grid.SetColumn(cameraBtn, 1);
                }
                else
                {
                    controlsGrid.HorizontalAlignment = HorizontalAlignment.Left;

                    Grid.SetColumn(controlsBtnPanel, 1);
                    Grid.SetColumn(cameraBtn, 0);
                }

                //Change the scrollbar visibility to hide the mulitple rows in list.
                if (imageScrollViewer != null)
                    imageScrollViewer.HorizontalScrollBarVisibility = ScrollBarVisibility.Disabled;

                try
                {
                    if (currentOrienationMode == ViewOrientationMode.OrientationModeLandScapeLeft)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(1);
                        }
                    }
                    else if (currentOrienationMode == ViewOrientationMode.OrientationModeLandScapeRight)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(3);
                        }
                    }

                }
                catch (DllNotFoundException Dexcp)
                {
                    MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
                }
                catch (FormatException fexcp)
                {
                    MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
                }
                catch (Exception excp)
                {
                    MessageBox.Show(excp.Message);
                }

                //Change the orientation image to landscape.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath + "landscape.png", UriKind.RelativeOrAbsolute));
                if (bi != null) orientationImage.Source = bi;

            }

            if (isFullscreenMode)
            {
                SwitchToFullscreenMode(currentOrienationMode);
            }

            //Set the image zoom as per the current orientation.
            ScaleImage(imgCurrentScaleX, imgCurrentScaleY);

            //Change the position for permission windows.
            double screenHeight = SystemParameters.PrimaryScreenHeight;
            double screenWidth = SystemParameters.PrimaryScreenWidth;

            if (accessCameraWindow != null && accessCameraWindow.isOpen)
            {
                accessCameraWindow.Width = screenWidth;
                accessCameraWindow.Top = screenHeight / 2;
            }

            if (accessLocationWindow != null && accessLocationWindow.isOpen)
            {
                accessLocationWindow.Width = screenWidth;
                accessLocationWindow.Top = screenHeight / 2;
            }
        }

        //Zoom / pan the center image.
        void ScaleImage(double scaleX, double scaleY)
        {
            Matrix borderMatrix = ((MatrixTransform)centerImgBorder.RenderTransform).Matrix;

            //Reset to original zoom.
            borderMatrix.M11 = 1.0;
            borderMatrix.M22 = 1.0;
            borderMatrix.OffsetX = 0;
            borderMatrix.OffsetY = 0;

            borderMatrix.ScaleAt(1.0,
                          1.0,
                          (centerImgBorder.Margin.Left + centerImgBorder.ActualWidth) / 2,
                          (centerImgBorder.Margin.Top + centerImgBorder.ActualHeight) / 2);
            centerImgBorder.RenderTransform = new MatrixTransform(borderMatrix);
            centerImgBorder.UpdateLayout();

            //Now set the required zoom.
            borderMatrix.ScaleAt(scaleX,
                           scaleY,
                           (centerImgBorder.Margin.Left + centerImgBorder.ActualWidth) / 2,
                           (centerImgBorder.Margin.Top + centerImgBorder.ActualHeight) / 2);
            centerImgBorder.RenderTransform = new MatrixTransform(borderMatrix);
            centerImgBorder.UpdateLayout();

            //set zoom level
            UpdateZoomLevel(borderMatrix.M11);

            //Store the current scale.
            imgCurrentScaleX = borderMatrix.M11;
            imgCurrentScaleY = borderMatrix.M22;
        }

        /// <summary>
        /// Main window loaded event handler.
        /// </summary>
        /// <param name="sender">Main window</param>
        /// <param name="e">Event arguments</param>
        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            //Initialize parameters
            isFullscreenMode = false;
            //

            isBatteryBelowThreshold = vm.IsBatteryBelowThreshold();

            double screenHeight = System.Windows.SystemParameters.PrimaryScreenHeight;
            double screenWidth = System.Windows.SystemParameters.PrimaryScreenWidth;

            this.imageViewMainGrid.Visibility = System.Windows.Visibility.Collapsed;
            this.webcamPreview.Visibility = System.Windows.Visibility.Visible;

            //Get the list box height.
            lbActualHeight = imagesListBox.ActualHeight;
            //Get the list box width.
            //lbActualWidth = imagesListBox.ActualWidth;

            //Create instance of PowerMngrHelper.
            powerHelper = new PowerMngrHelper(this);
            //Register for power source change notification.
            powerHelper.PowerSrcChanged += powerHelper_PowerSourceChanged;
            //Register for battery capacity change notification.
            powerHelper.BatteryPercentageChanged += powerHelper_BatteryCapacityChanged;
            //Find the scrollviewer component of ListBox. This ScrollViewer is used to page up/down on up/down arrow buttons.
            Border border = VisualTreeHelper.GetChild(imagesListBox as DependencyObject, 0) as Border;
            if (border != null)
            {
                imageScrollViewer = VisualTreeHelper.GetChild(border, 0) as ScrollViewer;
            }

            //layout the view as per current orientation.
            ChangeLayout(vm.viewCurrentOrientation);

            //Get the application name.
            applicationName = Application.Current.MainWindow.GetType().Assembly.GetName().Name;

            //Ask for permission to use camera.
            accessCameraWindow = new MessageWindow();
            accessCameraWindow.Width = screenWidth;
            accessCameraWindow.Top = screenHeight / 2;
            accessCameraWindow.Left = 0;
            accessCameraWindow.Message = String.Format("Can {0} use your camera?", applicationName);
            accessCameraWindow.allowButtonClicked += accessCameraWindow_allowButtonClicked;
            accessCameraWindow.cancelButtonClicked += accessCameraWindow_cancelButtonClicked;
            this.Opacity = 0.5;
            accessCameraWindow.ShowDialog();
            this.Opacity = 1.0;

            //Ask for permission to use location.
            accessLocationWindow = new MessageWindow();
            accessLocationWindow.Width = screenWidth;
            accessLocationWindow.Top = screenHeight / 2;
            accessLocationWindow.Left = 0;
            accessLocationWindow.Message = String.Format("Can {0} use your location?", applicationName);
            accessLocationWindow.allowButtonClicked += accessLocationWindow_allowButtonClicked;
            accessLocationWindow.cancelButtonClicked += accessLocationWindow_cancelButtonClicked;
            this.Opacity = 0.5;
            accessLocationWindow.ShowDialog();
            this.Opacity = 1.0;
        }

        void StartWebCam()
        {
            try
            {
                long result = WebCamHelper.Init(0);
                int myWidth = (int)this.webcamPreview.ActualWidth;
                int myHeight = (int)this.webcamPreview.ActualHeight;
                result = WebCamHelper.StartPreview(this.webcamPreview.Handle, 0, 0, myWidth, myHeight, true);
                shouldStartWebcam = false;
                //WebCamHelper.RotateVideo(5);
            }
            catch (DllNotFoundException Dexcp)
            {
                MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
            }
            catch (FormatException fexcp)
            {
                MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
            }
            catch (Exception excp)
            {
                MessageBox.Show(excp.Message);
            }
        }

        //Evnet invoked when user does not allow application to use gps location.
        void accessLocationWindow_cancelButtonClicked(object sender, RoutedEventArgs e)
        {
            //Disable the geo location panel.
            geoTagOnOffPnl.IsEnabled = false;

            //Stop the geo tag feature.
            vm.GeoLocationEnable = false;

            //Set this so that application should not automatically enables/disables the geo tag feature.
            vm.userClickedGeoTagBtn = true;

            //Set the flag.
            canAccessLocation = false;

            EnableLocation(false);
        }

        //Evnet invoked when user does not allow application to use device camera.
        void accessCameraWindow_cancelButtonClicked(object sender, RoutedEventArgs e)
        {
            //Disable camera.
            EnableCamera(false);

            //Set the flag.
            canAccessCamera = false;
        }

        //Evnet invoked when user allows application to use gps location.
        void accessLocationWindow_allowButtonClicked(object sender, RoutedEventArgs e)
        {
            //Start the geo tag feature.
            vm.GeoLocationEnable = true;

            //Set the flag.
            canAccessLocation = true;

            EnableLocation(true);
        }

        //Evnet invoked when user allows application to use device camera.
        void accessCameraWindow_allowButtonClicked(object sender, RoutedEventArgs e)
        {
            //Set the flag.
            canAccessCamera = true;

            //Check the battery percentage, if it is below threshold then do not start web cam.
            if (vm.IsBatteryBelowThreshold())
            {
                EnableCamera(false);
            }
            else
            {
                //Enable camera.
                EnableCamera(true);

                //Start webcam.
                StartWebCam();
            }
        }

        //Enables or disables the camera.
        void EnableCamera(bool enable)
        {
            if (enable)
            {
                cameraCaptureEnabled = true;
            }
            else
            {
                //Show the first image in list box
                if (vm.ImageList.Count > 0)
                {
                    if (imagesListBox.SelectedIndex < 0)
                        imagesListBox.SelectedIndex = 0;
                }
                else
                {
                    //Hide web cam view.
                    HideWebCamView();
                }

                cameraCaptureEnabled = false;
            }
        }

        //Enables or disables the location access.
        void EnableLocation(bool enable)
        {
            //Set the flag.
            locationAccessEnabled = enable;
        }

        /// <summary>
        /// MainWindow closing event handler.
        /// </summary>
        /// <param name="sender">Main window.</param>
        /// <param name="e">Event arguments.</param>
        void MainWindow_Closing(object sender, CancelEventArgs e)
        {
            //Release webcam resources
            try
            {
                long res = WebCamHelper.UnInit();
            }
            catch (DllNotFoundException Dexcp)
            {
                MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
            }
            catch (FormatException fexcp)
            {
                MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
            }
            catch (Exception excp)
            {
                MessageBox.Show(excp.Message);
            }

            //Unregister from power related notifications.
            powerHelper.PowerSrcChanged -= powerHelper_PowerSourceChanged;
            powerHelper.BatteryPercentageChanged -= powerHelper_BatteryCapacityChanged;

            //Call view model's method to unregister other handlers.
            vm.UnregisterHandlers();
        }

        /// <summary>
        /// Battery capacity changed event handler.
        /// </summary>
        /// <param name="sender">Battery</param>
        /// <param name="e">Event arguments.</param>
        void powerHelper_BatteryCapacityChanged(object sender, EventArgs e)
        {
            //Battery capacity changed, detect the current battery life & update value of "PowerState" property. This will automatically update UI because of property changed notification.
            vm.SetBatteryLife();
        }

        /// <summary>
        /// Power source changed event handler.
        /// </summary>
        /// <param name="sender">Power source.</param>
        /// <param name="e">Event arguments.</param>
        void powerHelper_PowerSourceChanged(object sender, EventArgs e)
        {
            //Power source changed, detect the current power source & update value of "PowerState" property. This will automatically update UI because of property changed notification.
            vm.SetPowerSource();
        }

        /// <summary>
        /// Selected event changed event handler.
        /// </summary>
        /// <param name="sender">Listbox</param>
        /// <param name="e">Event arguments.</param>
        private void ListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //Hide web cam view if visible.
            if (webcamPreview.Visibility == Visibility.Visible)
                HideWebCamView();

            //Set arrow images accordingly.
            SetArrowImages();

            //Update count.
            if (this.imagesListBox != null && vm.ImageList != null)
            {
                string currentbyTotal = (this.imagesListBox.SelectedIndex + 1).ToString() + "/" + vm.ImageList.Count;
                //SwitchToWindowMode();
                UpdateCurrentByTotalCount(currentbyTotal);
            }

            //Show large image for selected index.
            vm.ShowLargeImage(imagesListBox.SelectedIndex);

            //Set the selected image as per the current zoom factor
            ScaleImage(imgCurrentScaleX, imgCurrentScaleY);
        }

        void HideWebCamView()
        {
            //Show only if there is an image
            if (imagesListBox.SelectedIndex >= 0)
                this.ImageBottomBarPanel.Visibility = System.Windows.Visibility.Visible;

            this.webcamPreview.Visibility = System.Windows.Visibility.Collapsed;
            this.imageViewMainGrid.Visibility = System.Windows.Visibility.Visible;

            this.cameraBtn.IsEnabled = false;

            Matrix imgMatrix = ((MatrixTransform)this.centerImageView.RenderTransform).Matrix;
            UpdateZoomLevel(imgMatrix.M11);
            try
            {
                long result = WebCamHelper.StopPreview();
            }
            catch (DllNotFoundException Dexcp)
            {
                MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
            }
            catch (FormatException fexcp)
            {
                MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
            }
            catch (Exception excp)
            {
                MessageBox.Show(excp.Message);
            }
            if (this.imagesListBox != null && vm.ImageList != null)
            {
                string currentbyTotal = (this.imagesListBox.SelectedIndex + 1).ToString() + "/" + vm.ImageList.Count;
                UpdateCurrentByTotalCount(currentbyTotal);
            }

            //Hide the camera button.
            this.cameraBtn.Visibility = System.Windows.Visibility.Collapsed;
        }

        void UpdateCurrentByTotalCount(string strCount)
        {
            this.currentByTotalCountlbl.Content = strCount;
        }

        /// <summary>
        /// Down arrow button is clicked.
        /// </summary>
        /// <param name="sender">Button</param>
        /// <param name="e">Event arguments.</param>
        private void scrollDownBtn_Click(object sender, RoutedEventArgs e)
        {
            //ChangeLayout(ViewOrientationMode.OrientationModeLandScapeLeft);
            //Scroll down the listbox.
            if (imageScrollViewer != null)
            {
                //Portrait mode
                if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeFaceUp ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeFaceDown ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModePortraitUp ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModePortraitDown)
                {
                    imageScrollViewer.PageRight();
                }
                else if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeLeft ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeRight) //Landscape mode
                {
                    //imageScrollViewer.PageRight();
                    imageScrollViewer.PageDown();
                }
            }
        }

        /// <summary>
        /// Up arrow button is clicked.
        /// </summary>
        /// <param name="sender">Button</param>
        /// <param name="e">Event arguments.</param>
        private void scrollUpBtn_Click(object sender, RoutedEventArgs e)
        {
            //ChangeLayout(ViewOrientationMode.OrientationModePortraitUp);
            //Scroll up the listbox.
            if (imageScrollViewer != null)
            {
                //Portrait mode
                if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeFaceUp ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeFaceDown ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModePortraitUp ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModePortraitDown)
                {
                    imageScrollViewer.PageLeft();
                }
                else if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeLeft ||
                    vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeRight) //Landscape mode
                {
                    //imageScrollViewer.PageLeft();
                    imageScrollViewer.PageUp();
                }
            }
        }
        /// <summary>
        /// Debug button clicked, show debug panel.
        /// </summary>
        /// <param name="sender">Button.</param>
        /// <param name="e">Event arguments</param>
        private void debugBtn_Click(object sender, RoutedEventArgs e)
        {

            //If debug panel is already visible then hide it otherwise show it.
            if (debugPanel.Visibility == Visibility.Visible)
            {
                debugPanel.Visibility = Visibility.Collapsed;
                debugBtn.IsSelected = false;
            }
            else
            {
                debugPanel.Visibility = Visibility.Visible;
                debugBtn.IsSelected = true;
            }
        }

        /// <summary>
        /// Debug mode change button is clicked. If in graphical mode then switch to textual mode.
        /// If in textual mode then switch to graphical mode.
        /// </summary>
        /// <param name="sender">Button</param>
        /// <param name="e">Event arguments.</param>
        private void debugModeBtn_Click(object sender, RoutedEventArgs e)
        {
            if (debugPanel.Visibility == Visibility.Visible) //if debug panel is already visible
            {
                if (debugGraphicaPanel.Visibility == System.Windows.Visibility.Visible)
                {
                    debugGraphicaPanel.Visibility = System.Windows.Visibility.Collapsed; //Hide debug graphical panel
                }
                else
                {
                    debugGraphicaPanel.Visibility = System.Windows.Visibility.Visible; //Show debug graphical panel
                }
            }
        }

        /// <summary>
        /// Finds the items panel within item presenter, we have used this method to get the items panel for listbox.
        /// </summary>
        /// <param name="vs">Visual object from which we want to find the items panel.</param>
        /// <returns></returns>
        private object FindItemsPanel(Visual vs)
        {
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(vs); i++)
            {
                Visual child = VisualTreeHelper.GetChild(vs, i) as Visual;
                if (child != null)
                {
                    if (child is object && VisualTreeHelper.GetParent(child) is ItemsPresenter)
                    {
                        object temp = child;
                        return temp;
                    }

                    object panel = FindItemsPanel(child);
                    if (panel != null)
                    {
                        object temp = panel;
                        return temp;
                    }
                }
            }

            return default(object);
        }

        private void cameraBtn_Click(object sender, RoutedEventArgs e)
        {
            DataServices.DataService ds = new DataServices.DataService();
            if (String.IsNullOrEmpty(ds.PicturesDirectory))
                return;
            String timeStamp = DateTime.Now.ToLocalTime().ToLongTimeString();
            String ticks = DateTime.Now.Ticks.ToString();
            string imgName = "image" + ticks + ".bmp";
            string imageNameWithPath = ds.PicturesDirectory + "\\" + imgName;
            long re = WebCamHelper.CaptureImage(imageNameWithPath);

            if (re == 0)
            {
                Stream ms = null;
                System.Drawing.Image img = null;

                try
                {
                    ms = new FileStream(imageNameWithPath, FileMode.Open, FileAccess.ReadWrite);
                    img = System.Drawing.Image.FromStream(ms);
                }
                catch (FileNotFoundException fex)
                {
                    MessageBox.Show("File not found: " + fex.Message);
                }
                catch (Exception em)
                {
                    MessageBox.Show(em.Message);
                }

                if (img != null)
                {
                    try
                    {
                        //Convert to Jpeg
                        int ind = imageNameWithPath.LastIndexOf('.');

                        string jpegPath = imageNameWithPath.Substring(0, ind) + ".jpg";
                        img.Save(jpegPath, System.Drawing.Imaging.ImageFormat.Jpeg);
                        ms.Close();

                        //GeoTag the image.
                        //vm.GeoTagImage(jpegPath);

                        //Delete the bitmap file.
                        File.Delete(imageNameWithPath);
                    }
                    catch (UnauthorizedAccessException uex)
                    {
                        MessageBox.Show("UnAuthorized exception: " + uex.Message);
                    }
                    catch (Exception em)
                    {
                        MessageBox.Show(em.Message);
                    }
                }

                //refresh list view.
                vm.LoadImages();
                //Scroll to first item
                imagesListBox.ScrollIntoView(imagesListBox.Items[0]);
            }
        }

        private void webcamPreview_SizeChanged(object sender, RoutedEventArgs e)
        {
            int myWidth = (int)this.webcamPreview.ActualWidth;
            int myHeight = (int)this.webcamPreview.ActualHeight;
            try
            {
                long re = WebCamHelper.SetPreviewWindow(this.webcamPreview.Handle, 0, 0, myWidth, myHeight);
            }
            catch (DllNotFoundException Dexcp)
            {
                MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
            }
            catch (FormatException fexcp)
            {
                MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
            }
            catch (Exception excp)
            {
                MessageBox.Show(excp.Message);
            }
        }

        private void ImageViewClose_Click(object sender, RoutedEventArgs e)
        {
            //If user not permit to access camera.
            if (!canAccessCamera) return;

            //Check whether battery below threshold.
            if (isBatteryBelowThreshold && !cameraCaptureEnabled && !cameraAccepted)
            {
                //Ask user to enable.
                //if (MessageBox.Show("Battery is below threshold value, do you want to use camera?", applicationName, MessageBoxButton.YesNo) == MessageBoxResult.Yes)
                if (AskCameraPermission("Battery is low, do you want to enable camera capture?"))
                {
                    cameraAccepted = true;
                    EnableCamera(true);
                }
                else
                    return;
            }

            if (isFullscreenMode)
                SwitchToWindowMode();

            //Deselect image.
            imagesListBox.SelectedIndex = -1;

            this.imageViewMainGrid.Visibility = System.Windows.Visibility.Collapsed;
            this.webcamPreview.Visibility = System.Windows.Visibility.Visible;
            this.ImageBottomBarPanel.Visibility = System.Windows.Visibility.Collapsed;

            this.cameraBtn.IsEnabled = true;
            //Show the camera button.
            cameraBtn.Visibility = Visibility.Visible;

            try
            {
                int myWidth = (int)this.webcamPreview.ActualWidth;
                int myHeight = (int)this.webcamPreview.ActualHeight;
                if (shouldStartWebcam)
                {
                    StartWebCam();
                }
                else
                {
                    long result = WebCamHelper.RestartPreview();
                }
                try
                {
                    /*
                    ROTATE_NONE						= 0,
                    ROTATE_90_DEGREES_CLOCKWISE		= 1,
                    ROTATE_180_DEGREES_CLOCKWISE	= 2,
                    ROTATE_270_DEGREES_CLOCKWISE	= 3,
                    ROTATE_FLIP_VERTICAL			= 4,
                    ROTATE_FLIP_HORIZONTAL			= 5,
                    ROTATE_FLIP_DIAGONALLY			= 6
                    */
                    if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeFaceUp || vm.viewCurrentOrientation == ViewOrientationMode.OrientationModePortraitUp)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(0);
                        }
                    }
                    else if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeFaceDown || vm.viewCurrentOrientation == ViewOrientationMode.OrientationModePortraitDown)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(2);
                        }
                    }
                    if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeLeft)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(1);
                        }
                    }
                    else if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeRight)
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(3);
                        }
                    }
                    else if (vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeLeft || vm.viewCurrentOrientation == ViewOrientationMode.OrientationModeLandScapeRight) //LandScape mode
                    {
                        if (this.cameraBtn.Visibility == System.Windows.Visibility.Visible)
                        {
                            long res = WebCamHelper.RotateVideo(2);
                        }
                    }
                }//end try
                catch (DllNotFoundException Dexcp)
                {
                    MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
                }
                catch (FormatException fexcp)
                {
                    MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
                }
                catch (Exception excp)
                {
                    MessageBox.Show(excp.Message);
                }
            }//end try
            catch (DllNotFoundException Dexcp)
            {
                MessageBox.Show(AppMessages.DllNotFoundExceptionMessage + " " + "MoreDetails:" + Dexcp.Message);
            }
            catch (FormatException fexcp)
            {
                MessageBox.Show(AppMessages.DllFormatExceptionMessage + " " + "MoreDetails:" + fexcp.Message);
            }
            catch (Exception excp)
            {
                MessageBox.Show(excp.Message);
            }
        }

        private void Previous_Image_Click(object sender, RoutedEventArgs e)
        {
            if (this.imagesListBox != null)
            {
                int previousImage = this.imagesListBox.SelectedIndex - 1;
                if (previousImage >= 0)
                {
                    this.imagesListBox.SelectedIndex = previousImage;
                }

                SetArrowImages();
            }
        }

        private void Next_Image_Click(object sender, RoutedEventArgs e)
        {
            if (this.imagesListBox != null)
            {
                int nextImage = this.imagesListBox.SelectedIndex + 1;

                if (nextImage < vm.ImageList.Count)
                {
                    this.imagesListBox.SelectedIndex = nextImage;
                }

                SetArrowImages();
            }
        }

        private void SetArrowImages()
        {
            if (imagesListBox.SelectedIndex <= 0)
                prevImageBtn.IsEnabled = false;
            else
                prevImageBtn.IsEnabled = true;

            if (imagesListBox.SelectedIndex >= vm.ImageList.Count - 1)
                nextImageBtn.IsEnabled = false;
            else
                nextImageBtn.IsEnabled = true;
        }

        //Invoked when geo location button clicked.
        private void geoTagBtn_Click(object sender, RoutedEventArgs e)
        {
            //If user not permit to access location.
            if (!canAccessLocation) return;

            //Check whether battery below threshold.
            if (isBatteryBelowThreshold && !locationAccessEnabled && !locationAccepted)
            {
                //Ask user to enable.
                if (AskLocationPermission("Battery is low, do you want to enable location access?"))
                {
                    //ToastMsgHelper.ShowToastMessage(AppConstants.EnableLocation);
                    locationAccepted = true;
                    EnableLocation(true);
                }
                else
                    return;
            }

            vm.GeoLocationEnable = !vm.GeoLocationEnable;

            //User clicked geo tag button so now we will not automatically enable or disable it.
            vm.userClickedGeoTagBtn = true;
        }

        //Manipulation evvents
        private void centerImageView_ManipulationStarting(object sender, System.Windows.Input.ManipulationStartingEventArgs e)
        {
            e.ManipulationContainer = this;
            e.Handled = true;
        }

        private void centerImageView_ManipulationInertiaStarting(object sender, System.Windows.Input.ManipulationInertiaStartingEventArgs e)
        {
            //Set the manipulations inertia values

            e.ExpansionBehavior = new InertiaExpansionBehavior()
            {
                InitialVelocity = e.InitialVelocities.ExpansionVelocity,
                DesiredDeceleration = 10.0 * 96.0 / 1000000.0
            };

            e.Handled = true;
        }

        private void centerImageView_ManipulationStarted(object sender, System.Windows.Input.ManipulationStartedEventArgs e)
        {

        }

        private void centerImageView_ManipulationDelta(object sender, System.Windows.Input.ManipulationDeltaEventArgs e)
        {
            //Scale Up/Down
            if (!isFullscreenMode)  //Allow to zoom and pinch only in non full screen mode.
            {
                Matrix borderMatrix = ((MatrixTransform)centerImgBorder.RenderTransform).Matrix;

                var maxScale = Math.Max(e.DeltaManipulation.Scale.X, e.DeltaManipulation.Scale.Y);

                if ((maxScale < 1 && borderMatrix.M11 * maxScale > AppConstants.MinimumZoom) || //< 1 == pinch
                    (maxScale > 1 && borderMatrix.M11 * maxScale < AppConstants.MaximumZoom)) // > 1  == zoom
                {

                    borderMatrix.ScaleAt(e.DeltaManipulation.Scale.X,
                                    e.DeltaManipulation.Scale.Y,
                                    (centerImgBorder.ActualWidth / 2),
                                    (centerImgBorder.ActualHeight / 2));
                    centerImgBorder.RenderTransform = new MatrixTransform(borderMatrix);
                    UpdateZoomLevel(borderMatrix.M11);

                    //Store the current scale.
                    imgCurrentScaleX = borderMatrix.M11;
                    imgCurrentScaleY = borderMatrix.M22;

                    //ScaleImage(e.DeltaManipulation.Scale.X, e.DeltaManipulation.Scale.Y);

                    e.Handled = true;
                    if (e.IsInertial)
                    {
                        //Get the containing elements size rect
                        Rect containingRect = new Rect(((FrameworkElement)e.ManipulationContainer).RenderSize);
                        //Get the transformed elements new bounds
                        Rect shapeBounds = centerImgBorder.RenderTransform.TransformBounds(new Rect(centerImgBorder.RenderSize));
                        //If elements falls out of bounds
                        if (!containingRect.Contains(shapeBounds))
                        {
                            //Report boundary feedback
                            e.ReportBoundaryFeedback(e.DeltaManipulation);
                            //Stop any afther inertia
                            e.Complete();
                        }
                    }
                }
            }

            //Get translation data for swipe gesture
            isManipulated = true;
            cumulativeDeltaX = e.CumulativeManipulation.Translation.X;
            cumulativeDeltaY = e.CumulativeManipulation.Translation.Y;
            linearVelocity = e.Velocities.LinearVelocity.X;
            if (linearVelocity < 0)
                linearVelocity = (-1) * linearVelocity;
        }

        private void SwitchToFullscreenMode(ViewOrientationMode currentOrienttaion)
        {
            //Set the stretch for center image view.
            centerImageView.Stretch = Stretch.Fill;

            fullScreenBtn.IsSelected = true;
            isFullscreenMode = true;

            this.bottomGrid.Visibility = System.Windows.Visibility.Collapsed;
            this.debugPanel.Visibility = System.Windows.Visibility.Collapsed;

            //Hide the custom titlebar.
            titleBar.Visibility = Visibility.Collapsed;

            ScaleImage(1.0, 1.0);

            //As per the requirement, only show the prev/next image buttons in full screen mode.
            prevImageBtn.Visibility = Visibility.Visible;
            nextImageBtn.Visibility = Visibility.Visible;
            currentByTotalCountlbl.Visibility = Visibility.Visible;

            //As per the requirement, dont show the +/- buttons in full screen mode.
            zoomInBtn.Visibility = Visibility.Collapsed;
            zoomOutBtn.Visibility = Visibility.Collapsed;
            zoomLevel.Visibility = Visibility.Collapsed;
        }

        private void SwitchToWindowMode()
        {
            this.bottomGrid.Visibility = System.Windows.Visibility.Visible;

            if (debugBtn.IsSelected)
                this.debugPanel.Visibility = System.Windows.Visibility.Visible;
            else
                this.debugPanel.Visibility = System.Windows.Visibility.Collapsed;

            this.centerImageView.IsManipulationEnabled = true;

            //Set the stretch for center image view.
            centerImageView.Stretch = Stretch.Uniform;

            //Show the custom titlebar.
            titleBar.Visibility = Visibility.Visible;

            fullScreenBtn.IsSelected = false;
            isFullscreenMode = false;

            //As per the requirement, no need to show the prev/next image buttons in window mode.
            prevImageBtn.Visibility = Visibility.Collapsed;
            nextImageBtn.Visibility = Visibility.Collapsed;
            currentByTotalCountlbl.Visibility = Visibility.Collapsed;

            //As per the requirement, show the +/- buttons in non full screen mode only.
            zoomInBtn.Visibility = Visibility.Visible;
            zoomOutBtn.Visibility = Visibility.Visible;
            zoomLevel.Visibility = Visibility.Visible;
        }

        private void centerImageView_ManipulationCompleted(object sender, System.Windows.Input.ManipulationCompletedEventArgs e)
        {
            if (!isManipulated) //if manipulation delta is not called that means no manipulation, just touch down & up
            {
                //AppLogger.LogMessage(debugTextBlock, "isManipulated = false, so returning.");  -- Commented 31Jul2012
                return;
            }
            double unsignedDeltaX;
            double unsignedDeltaY;
            bool isRightToLeftSwipe = false;

            if (cumulativeDeltaX < 0) // moving from right to left
            {
                isRightToLeftSwipe = true;
                unsignedDeltaX = (-1) * cumulativeDeltaX;
            }
            else
            {
                unsignedDeltaX = cumulativeDeltaX;
            }

            if (cumulativeDeltaY < 0)
            {
                unsignedDeltaY = (-1) * cumulativeDeltaY;
            }
            else
            {
                unsignedDeltaY = cumulativeDeltaY;
            }

            if (isSwipeGesture(unsignedDeltaX, unsignedDeltaY, linearVelocity)) //this is swipe gesture
            {
                int currentIndex = this.imagesListBox.SelectedIndex;
                if (isRightToLeftSwipe) //right to left
                {
                    currentIndex += 1;
                }
                else //left to right
                {
                    currentIndex -= 1;
                }

                if (this.imagesListBox != null && vm.ImageList != null && currentIndex >= 0 && currentIndex < vm.ImageList.Count)
                {
                    this.imagesListBox.SelectedIndex = currentIndex;
                }
                e.Handled = false;
            }

            isManipulated = false;
        }

        private bool isSwipeGesture(double deltaX, double deltaY, double linearVelocity)
        {
            bool result = false;
            if (deltaY <= AppConstants.DeltaY && deltaX >= AppConstants.DeltaX && linearVelocity >= AppConstants.LinearVelocityX) //deltaX & deltaY is good to go
            {
                result = true;
            }
            return result;
        }

        private void FullSCreenModeButton_Click(object sender, RoutedEventArgs e)
        {

            if (isFullscreenMode)
            {
                SwitchToWindowMode();
            }
            else
            {
                SwitchToFullscreenMode(vm.viewCurrentOrientation);
            }

        }

        private void UpdateZoomLevel(double curZoomLevel)
        {
            curZoomLevel = curZoomLevel * 100;
            uint zoomLevel = Convert.ToUInt32(curZoomLevel);
            this.zoomLevel.Content = zoomLevel.ToString() + " %";
        }

        private void Minus_Click(object sender, RoutedEventArgs e)
        {
            if (isFullscreenMode)
                return;
            ZoomOut();
        }

        private void Plus_Click(object sender, RoutedEventArgs e)
        {
            if (isFullscreenMode)
                return;
            ZoomIn();
        }

        //Zooms in the image.
        private void ZoomIn()
        {
            Matrix borderMatrix = ((MatrixTransform)centerImgBorder.RenderTransform).Matrix;

            double curScaleX = borderMatrix.M11;
            double curScaleY = borderMatrix.M22;

            if (borderMatrix.M11 <= AppConstants.MaximumZoom)
            {
                curScaleX = borderMatrix.M11 + AppConstants.ZoomFactor;
                curScaleY = borderMatrix.M11 + AppConstants.ZoomFactor;
                if (curScaleX > AppConstants.MaximumZoom)
                {
                    curScaleX = AppConstants.MaximumZoom;
                }
                if (curScaleY > AppConstants.MaximumZoom)
                {
                    curScaleY = AppConstants.MaximumZoom;
                }

                ScaleImage(curScaleX, curScaleY);
            }

        }

        //Zooms out (pan) the image.
        private void ZoomOut()
        {
            Matrix borderMatrix = ((MatrixTransform)centerImgBorder.RenderTransform).Matrix;

            double curScaleX = borderMatrix.M11;
            double curScaleY = borderMatrix.M22;

            if (borderMatrix.M11 >= AppConstants.MinimumZoom)
            {
                curScaleX = borderMatrix.M11 - AppConstants.ZoomFactor;
                curScaleY = borderMatrix.M22 - AppConstants.ZoomFactor;

                if (curScaleX < AppConstants.MinimumZoom)
                {
                    curScaleX = AppConstants.MinimumZoom;
                }
                if (curScaleY < AppConstants.MinimumZoom)
                {
                    curScaleY = AppConstants.MinimumZoom;
                }

                ScaleImage(curScaleX, curScaleY);

            }
        }

        private void minimizeBtn_Click(object sender, RoutedEventArgs e)
        {
            this.WindowState = WindowState.Minimized;
        }

        private void closeBtn_Click(object sender, RoutedEventArgs e)
        {
            this.Close();
        }

        private void fullScreenBtn_MouseEnter(object sender, MouseEventArgs e)
        {
            Image btnImage = VisualTreeHelper.GetChild(fullScreenBtn, 0) as Image;
            if (btnImage == null) return;

            if (isFullscreenMode)
            {
                //Change the hover image for full screeen.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath + "full-screen-exit-hover.png", UriKind.RelativeOrAbsolute));
                if (bi != null) btnImage.Source = bi;
            }
            else
            {
                //Change the hover image for window screeen.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath + "full-screen-hover.png", UriKind.RelativeOrAbsolute));
                if (bi != null) btnImage.Source = bi;
            }
        }

        private void fullScreenBtn_MouseLeave(object sender, MouseEventArgs e)
        {
            Image btnImage = VisualTreeHelper.GetChild(fullScreenBtn, 0) as Image;
            if (btnImage == null) return;

            if (isFullscreenMode)
            {
                //Change the normal image for full screeen.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath + "full-screen-exit.png", UriKind.RelativeOrAbsolute));
                if (bi != null) btnImage.Source = bi;
            }
            else
            {
                //Change the normal image for window screeen.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath + "full-screen.png", UriKind.RelativeOrAbsolute));
                if (bi != null) btnImage.Source = bi;
            }
        }
    }
}
