﻿/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.
 
The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.
 
Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intel’s suppliers or licensors in any way.

*/

using System;
using System.Collections.Generic;
using System.Management;
using System.Diagnostics;
using System.Windows;

namespace PhotoApp.Utility
{
    /// <summary>
    /// Provides methods to get or set the current display brightness. Also provides notifications when value for current brightness changes.
    /// Used WMI (Windows Management Instrumentation) classes for brightness.
    /// </summary>
    class DisplayBrightnessHelper
    {
        //Supported brightness leveles in percentage.
        static byte[] brightnessLevels;
        //WmiMonitorBrightnessMethods object.
        private static ManagementObject brightnessMethodsObject = null;
        // List of event handlers registered to get the brightness change event.
        private List<EventHandler> eventHandlers = new List<EventHandler>();
        //ManagementEventWatcher to watch the brightness change event.
        ManagementEventWatcher brightnessEventWathcer;

        /// <summary>
        /// Gets the supported brightness values for the display in percentages.
        /// </summary>
        /// <returns>Returns array of supported brightness values if .</returns>
        private static byte[] GetBrightnessLevels()
        {
            byte[] brightness = null;

            //Initialize the ManagementScope with path to WMI classes.
            ManagementScope scope = new ManagementScope("root\\WMI");
            scope.Connect();

            try
            {
                //Query to get the WmiMonitorBrightness instance.
                List<ManagementObject> BrightnessObjects = GetWMIResult(scope, "SELECT * from WmiMonitorBrightness", false);

                //If valid result.
                if (BrightnessObjects != null && BrightnessObjects.Count > 0)
                {
                    //Get the "Level" property. It is the array of supported levels in percentage.
                    ManagementObject BrightnessObject = BrightnessObjects[0];
                    brightness = (byte[])BrightnessObject["Level"];

                    //Query to get the WmiMonitorBrightnessMethods instance. This will be used while setting the brightness.
                    List<ManagementObject> BrightnessMethodObjects = GetWMIResult(scope, "SELECT * from WmiMonitorBrightnessMethods", false);
                    brightnessMethodsObject = BrightnessMethodObjects[0];
                }
            }
            catch (NotSupportedException nse)
            {
                //Not supported.
                Debug.Print("GetBrightnessLevels Excep:" + nse.Message);
            }

            return brightness;
        }

        /// <summary>
        /// Gets the current display brightness in percentage.
        /// </summary>
        /// <returns>Returns current display brightness in percentage.</returns>
        public static byte GetCurrentBrightness()
        {
            byte currentBrightness = 0;

            //Initialize the ManagementScope with path to WMI classes.
            ManagementScope scope = new ManagementScope("root\\WMI");
            scope.Connect();

            try
            {
                //Query to get the WmiMonitorBrightness instance.
                List<ManagementObject> BrightnessObjects = GetWMIResult(scope, "SELECT * from WmiMonitorBrightness", false);

                //If valid result.
                if (BrightnessObjects != null && BrightnessObjects.Count > 0)
                {
                    //Get the "CurrentBrightness" property.
                    ManagementObject BrightnessObject = BrightnessObjects[0];
                    currentBrightness = (byte)BrightnessObject["CurrentBrightness"];
                }
            }
            catch (NotSupportedException nse)
            {
                //Not supported.
                Debug.Print("GetBrightnessLevels Excep:" + nse.Message);
            }

            return currentBrightness;
        }

        /// <summary>
        /// Set the current display brightness.
        /// </summary>
        /// <param name="Percent">Brightness value to set in percentage.</param>
        public static void SetBrightness(int Percent)
        {
            //Get the supported brightness levels.
            if (brightnessLevels == null)
            {
                brightnessLevels = GetBrightnessLevels();
            }

            //If not able to get the brightness levels then return.
            if (brightnessLevels == null) return;

            //Percentage value should not be greater than 100.
            Percent = Math.Min(100, Math.Max(0, Percent));

            //Initialize the ManagementScope with path to WMI classes.
            ManagementScope scope = new ManagementScope("root\\WMI");
            scope.Connect();

            //Get the nearest supported values to given value.
            int ClosestBrightness = brightnessLevels[0];
            foreach (int level in brightnessLevels)
            {
                if (Math.Abs(level - Percent) < Math.Abs(ClosestBrightness - Percent))
                    ClosestBrightness = level;
            }

            if (ClosestBrightness != Percent)
                Debug.Print("Closest brightness to {0}% is {1}%.", Percent, ClosestBrightness);

            //Invoke WmiSetBrightness and set the brightness.
            brightnessMethodsObject.InvokeMethod("WmiSetBrightness", new object[] { (Int64)0, (byte)ClosestBrightness });
        }

        /// <summary>
        /// EventHandler to receive the brightness changes.
        /// </summary>
        public event EventHandler BrightnessChanged
        {
            add
            {
                //Start the event watcher.
                if (eventHandlers.Count <= 0)
                    RegisterBrightnessChange();

                eventHandlers.Add(value);
            }
            remove
            {
                eventHandlers.Remove(value);

                //Stop the event watcher.
                UnregisterBrightnessChange();
            }
        }

        /// <summary>
        /// Register to get the brightness change notification.
        /// </summary>
        private void RegisterBrightnessChange()
        {
            //Initialize the ManagementScope with path to WMI classes.
            ManagementScope scope = new ManagementScope("root\\WMI");
            scope.Connect();

            //Brigtness change event query.
            EventQuery eQuery = new EventQuery("select * from WmiMonitorBrightnessEvent");

            //Just stop if previous running.
            UnregisterBrightnessChange();

            //Create event watcher object to watch the brightness change event.
            brightnessEventWathcer = new ManagementEventWatcher(scope,eQuery);
            brightnessEventWathcer.EventArrived += brightnessEventWathcer_EventArrived;
            brightnessEventWathcer.Start();
        }

        /// <summary>
        /// Unregister from receiving the brightness change notification.
        /// </summary>
        private void UnregisterBrightnessChange()
        {
            //Stop the event watcher.
            if (brightnessEventWathcer != null)
                brightnessEventWathcer.Stop();
        }

        /// <summary>
        /// Notification method, called when brightness changed.
        /// </summary>
        /// <param name="sender">ManagementEventWatcher as sender</param>
        /// <param name="e">Argumets EventArrivedEventArgs</param>
        void brightnessEventWathcer_EventArrived(object sender, EventArrivedEventArgs e)
        {
            //Brightness changed; send notification to each handler.
            foreach (EventHandler eHandler in eventHandlers)
                eHandler.Invoke(null, new EventArgs());
        }

        /// <summary>
        /// Fires the WQL query and returns the result.
        /// </summary>
        /// <param name="scope">ManagementScope</param>
        /// <param name="query">Query to be fired.</param>
        /// <param name="noisy">If value true then prints the debug values, otherwise not.</param>
        /// <returns>Returns list of ManagementObject as a query result.</returns>
        static List<ManagementObject> GetWMIResult(ManagementScope scope, string query, bool noisy)
        {
            //Remove the escape "\" character.
            query = query.Replace(@"\", @"\\");
            if (noisy)
            {
                Debug.Print("Executing query: " + query);
                Debug.Flush();
            }

            //Searcher object.
            ManagementObjectSearcher WMI;
            if (scope == null)
                WMI = new ManagementObjectSearcher(query);
            else
                WMI = new ManagementObjectSearcher(scope, new ObjectQuery(query));

            //Get the object collection from searcher.
            ManagementObjectCollection objectCollection = WMI.Get();
            List<ManagementObject> objects = new List<ManagementObject>();
            try
            {
                //Copy values from collection to list to return.
                objects.Capacity = objectCollection.Count;
                foreach (ManagementObject o in objectCollection)
                    objects.Add(o);
            }
            catch (ManagementException me)
            {
                if (me.ErrorCode == ManagementStatus.NotSupported)
                {
                    throw new NotSupportedException(me.Message, me);
                }
                if (noisy)
                {
                    Debug.Print("Management Exception: " + me.ToString());
                    Debug.Flush();
                }
                objects = null;
            }
            catch (Exception ex)
            {
                if (noisy)
                {
                    Debug.Print("Exception: " + ex.ToString());
                    Debug.Flush();
                }
                objects = null;
            }

            return objects;
        }
    }
}
