﻿/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.
 
The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.
 
Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intel’s suppliers or licensors in any way.

*/

using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Interop;
using System.Runtime.InteropServices;
using System.Collections;
using System.Management;

namespace PhotoApp.Utility
{
    public struct SYSTEM_POWER_STATUS
    {
        public ACLineStatus LineStatus;
        public BatteryFlag flgBattery;
        public Byte BatteryLifePercent;
        public Byte Reserved1;
        public Int32 BatteryLifeTime;
        public Int32 BatteryFullLifeTime;
    }
    public enum ACLineStatus : byte
    {
        Offline = 0,
        Online = 1,
        Unknown = 255
    }

    public enum BatteryFlag : byte
    {
        High = 1,
        Low = 2,
        Critical = 4,
        Charging = 8,
        NoSystemBattery = 128,
        Unknown = 255
    }

    public enum POWER_DATA_ACCESSOR
    {
        ACCESS_AC_POWER_SETTING_INDEX = 0,   // 0x0
        ACCESS_DC_POWER_SETTING_INDEX = 1,   // 0x1
        ACCESS_SCHEME = 16,  // 0x10
        ACCESS_SUBGROUP = 17,  // 0x11
        ACCESS_INDIVIDUAL_SETTING = 18,  // 0x12
        ACCESS_ACTIVE_SCHEME = 19,  // 0x13
        ACCESS_CREATE_SCHEME = 20   // 0x14
    }


    // This structure is used when the PBT_POWERSETTINGSCHANGE message is sent
    public struct PowerSetting
    {
        public Guid PowerSettingChanged;
        public Int32 DataLength;
    }

    public class PowerMngrHelper
    {
        //GUID for power source.                    //5d3e9a59-e9D5-4b00-a6bd-ff34ff516548
        internal static readonly Guid gPowerSourceChanged = new Guid(0x5d3e9a59, 0xe9D5, 0x4b00, 0xa6, 0xbd, 0xff, 0x34, 0xff, 0x51, 0x65, 0x48);

        //GUID for remaining battery percentage.    //a7ad8041-b45a-4cae-87a3-eecbb468a9e1
        internal static readonly Guid gBatteryCapacityChanged = new Guid(0xa7ad8041, 0xb45a, 0x4cae, 0x87, 0xa3, 0xee, 0xcb, 0xb4, 0x68, 0xa9, 0xe1);

        //Message ID for power broadcast message.
        const uint PowerBroadcastMsg = 536;
        //Message ID for power settting change message. 
        const uint PowerSettingChangeMsg = 32787;

        //Handler to the power source change notification.
        IntPtr hPowerSrc;
        //Handler to the battery capacity change notification.
        IntPtr hBatteryCapacity;
        //Number of events registered for notification.
        int eventCount = 0;

        //Interop Helper used to get the handle to WPF window.
        WindowInteropHelper interopHelper;

        //Stores the event handlers.
        List<EventHandler> eventsList = new List<EventHandler>();

        //Method to get the power information.
        [DllImport("powrprof.dll", SetLastError = true)]
        private static extern UInt32 CallNtPowerInformation(
             Int32 InformationLevel,
             IntPtr lpInputBuffer,
             UInt32 nInputBufferSize,
             IntPtr lpOutputBuffer,
             UInt32 nOutputBufferSize
             );

        /// <summary>
        /// Enumerates the specified elements in a power scheme. This function
        /// is normally called in a loop incrementing the Index parameter to
        /// retrieve subkeys until they've all been enumerated.
        /// </summary>
        /// <param name="RootPowerKey">This parameter is reserved for future use and must be set to NULL</param>
        /// <param name="SchemeGuid">The identifier of the power scheme. If this parameter is NULL,
        /// an enumeration of the power policies is returned</param>
        /// <param name="SubGroupOfPowerSettingsGuid">The subgroup of power settings. If this parameter is NULL,
        /// an enumeration of settings under the PolicyGuid key is returned</param>
        /// <param name="AccessFlags">A set of flags that specifies what will be enumerated</param>
        /// <param name="Index">The zero-based index of the scheme, subgroup, or setting that is being enumerated</param>
        /// <param name="Buffer">A pointer to a variable to receive the elements. If this parameter is NULL,
        /// the function retrieves the size of the buffer required</param>
        /// <param name="BufferSize">A pointer to a variable that on input contains the size of the buffer pointed to
        /// by the Buffer parameter. If the Buffer parameter is NULL or if the BufferSize is not large enough, the
        /// function will return ERROR_MORE_DATA and the variable receives the required buffer size</param>
        /// <returns>Returns ERROR_SUCCESS (zero) if the call was successful, and a non-zero value if the call failed.
        /// If the buffer size passed in the BufferSize parameter is too small, or if the Buffer parameter is NULL,
        /// ERROR_MORE_DATA will be returned and the DWORD pointed to by the BufferSize parameter will be filled in with
        /// the required buffer size.</returns>
        [DllImport("powrprof.dll", EntryPoint = "PowerEnumerate", CharSet = CharSet.Auto, SetLastError = true)]
        private static extern uint PowerEnumerate(IntPtr RootPowerKey, IntPtr SchemeGuid, IntPtr SubGroupOfPowerSettingsGuid, POWER_DATA_ACCESSOR AccessFlags, uint Index, ref Guid Buffer, ref uint BufferSize);

        /// <summary>
        /// Retrieves the friendly name for the specified power setting, subgroup, or scheme. If the SchemeGuid parameter is not NULL 
        /// but both the SubGroupOfPowerSettingsGuid and PowerSettingGuid parameters are NULL, the friendly name of the power scheme 
        /// will be returned. If the SchemeGuid and SubGroupOfPowerSettingsGuid parameters are not NULLand the PowerSettingGuid parameter
        /// is NULL, the friendly name of the subgroup will be returned. If the SchemeGuid, SubGroupOfPowerSettingsGuid, and PowerSettingGuid
        /// parameters are not NULL, the friendly name of the power setting will be returned.
        /// </summary>
        /// <param name="RootPowerKey">This parameter is reserved for future use and must be set to NULL.</param>
        /// <param name="SchemeGuid">The identifier of the power scheme.</param>
        /// <param name="SubGroupOfPowerSettingsGuid">The subgroup of power settings. Use NO_SUBGROUP_GUID to refer to the default power scheme.</param>
        /// <param name="PowerSettingGuid">The identifier of the power setting that is being used.</param>
        /// <param name="Buffer">A pointer to a buffer that receives the friendly name. If this parameter is NULL, the BufferSize parameter receives
        /// the required buffer size. The strings returned are all wide (Unicode) strings.</param>
        /// <param name="BufferSize">A pointer to a variable that contains the size of the buffer pointed to by the Buffer parameter.</param>
        /// <returns>Returns ERROR_SUCCESS (zero) if the call was successful, and a nonzero value if the call failed. If the buffer size specified
        /// by the BufferSize parameter is too small, ERROR_MORE_DATA will be returned and the DWORD pointed to by the BufferSize parameter will be
        /// filled in with the required buffer size.</returns>
        [DllImport("powrprof.dll", EntryPoint = "PowerReadFriendlyName")]
        private static extern uint PowerReadFriendlyName(IntPtr RootPowerKey, ref Guid SchemeGuid, IntPtr SubGroupOfPowerSettingsGuid, IntPtr PowerSettingGuid, IntPtr Buffer, ref uint BufferSize);

        /// <summary>
        /// Sets the active power scheme for the current user.
        /// </summary>
        /// <param name="UserPowerKey">This parameter is reserved for future use and must be set to NULL.</param>
        /// <param name="ActivePolicyGuid">The identifier of the power scheme.</param>
        /// <returns>Returns ERROR_SUCCESS (zero) if the call was successful, and a nonzero value if the call failed.</returns>
        [DllImport("powrprof.dll", EntryPoint = "PowerSetActiveScheme")]
        private static extern uint PowerSetActiveScheme(IntPtr UserPowerKey, ref Guid ActivePolicyGuid);

        /// <summary>
        /// Retrieves the active power scheme and returns a GUID that identifies the scheme.
        /// </summary>
        /// <param name="UserRootPowerKey">[in, optional] This parameter is reserved for future use and must be set to NULL</param>
        /// <param name="ActivePolicyGuid">[out] A pointer that receives a pointer to a GUID structure. Use the LocalFree function to free this memory.</param>
        /// <returns>Returns ERROR_SUCCESS (zero) if the call was successful, and a non-zero value if the call failed.</returns>
        [DllImport("powrprof.dll", EntryPoint = "PowerGetActiveScheme")]
        public static extern uint PowerGetActiveScheme(IntPtr UserRootPowerKey, out IntPtr ActivePolicyGuid);

        [DllImport("kernel32.dll", SetLastError = true)]
        private static extern Boolean GetSystemPowerStatus(out SYSTEM_POWER_STATUS sps);

        [DllImport("User32", SetLastError = true,
           EntryPoint = "RegisterPowerSettingNotification",
           CallingConvention = CallingConvention.StdCall)]
        private static extern IntPtr RegisterPowerSettingNotification(
                IntPtr hRecipient,
                ref Guid PowerSettingGuid,
                Int32 Flags);

        [DllImport("User32", SetLastError = true,
          EntryPoint = "UnregisterPowerSettingNotification",
          CallingConvention = CallingConvention.StdCall)]
        private static extern bool UnregisterPowerSettingNotification(
                IntPtr hRecipient);

        private Hashtable eventsHashTable = new Hashtable();

        public static SYSTEM_POWER_STATUS GetPowerState()
        {
            SYSTEM_POWER_STATUS powerStatus = new SYSTEM_POWER_STATUS();
            bool b = GetSystemPowerStatus(out powerStatus);

            return powerStatus;
        }
        /*public static SYSTEM_BATTERY_STATE GetBatteryState()
        {
            IntPtr status = Marshal.AllocCoTaskMem(Marshal.SizeOf(typeof(SYSTEM_BATTERY_STATE)));
            uint retval = CallNtPowerInformation(
              5,  // SystemBatteryState
              (IntPtr)null,
              0,
              status,
              (UInt32)Marshal.SizeOf(typeof(SYSTEM_BATTERY_STATE))
              );

            SYSTEM_BATTERY_STATE batt_status = (SYSTEM_BATTERY_STATE)Marshal.PtrToStructure(status, typeof(SYSTEM_BATTERY_STATE));

            return batt_status;
        }*/

        //Method to register for power source change notification.
        void RegisterPowerSourceChangeNotification(IntPtr aHandle)
        {
            //Add message hook
            if (eventCount <= 0)
            {
                HwndSource source = HwndSource.FromHwnd(interopHelper.Handle);
                source.AddHook(WindowProc);
            }

            eventCount++;

            Guid g = gPowerSourceChanged;
            hPowerSrc = RegisterPowerSettingNotification(aHandle, ref g, 0);
        }

        //Method to unregister for power source change notification.
        void UnregisterPowerSourceChangeNotification()
        {
            UnregisterPowerSettingNotification(hPowerSrc);

            eventCount--;

            //Remove message hook
            if (eventCount <= 0)
            {
                HwndSource src = HwndSource.FromHwnd(interopHelper.Handle);
                src.RemoveHook(WindowProc);
            }
        }

        //Method to register for battery capacity change notification.
        void RegisterBatteryCapacityChangeNotification(IntPtr aHandle)
        {
            //Add message hook
            if (eventCount <= 0)
            {
                HwndSource source = HwndSource.FromHwnd(interopHelper.Handle);
                source.AddHook(WindowProc);
            }

            eventCount++;

            Guid g = gBatteryCapacityChanged;
            hBatteryCapacity = RegisterPowerSettingNotification(aHandle, ref g, 0);
        }

        //Method to unregister for battery capacity change notification.
        void UnRegisterBatteryCapacityChangeNotification()
        {
            UnregisterPowerSettingNotification(hBatteryCapacity);

            eventCount--;

            //Remove message hook
            if (eventCount <= 0)
            {
                HwndSource src = HwndSource.FromHwnd(interopHelper.Handle);
                src.RemoveHook(WindowProc);
            }
        }

        /// <summary>
        /// Notification called when power source changed (i.e. from AC power to Battery power or vice versa)
        /// </summary>
        public event EventHandler PowerSrcChanged
        {
            add
            {
                //readerWriterLock.AcquireWriterLock(Timeout.Infinite);
                if (!eventsHashTable.Contains(gPowerSourceChanged))
                {
                    RegisterPowerSourceChangeNotification(interopHelper.Handle);
                    ArrayList newList = new ArrayList();
                    newList.Add(value);
                    eventsHashTable.Add(gPowerSourceChanged, newList);
                }
                else
                {
                    ArrayList currList = (ArrayList)eventsHashTable[gPowerSourceChanged];
                    currList.Add(value);
                }
                //readerWriterLock.ReleaseWriterLock();
            }
            remove
            {
                UnregisterPowerSourceChangeNotification();

                ArrayList currList = (ArrayList)eventsHashTable[gPowerSourceChanged];
                currList.Remove(value);
            }
        }

        /// <summary>
        /// Notification called when battery capacity changed.
        /// </summary>
        public event EventHandler BatteryPercentageChanged
        {
            add
            {
                // readerWriterLock.AcquireWriterLock(Timeout.Infinite);
                if (!eventsHashTable.Contains(gBatteryCapacityChanged))
                {
                    RegisterBatteryCapacityChangeNotification(interopHelper.Handle);
                    ArrayList newList = new ArrayList();
                    newList.Add(value);
                    eventsHashTable.Add(gBatteryCapacityChanged, newList);
                }
                else
                {
                    ArrayList currList = (ArrayList)eventsHashTable[gBatteryCapacityChanged];
                    currList.Add(value);
                }
                //readerWriterLock.ReleaseWriterLock();
            }
            remove
            {
                UnRegisterBatteryCapacityChangeNotification();

                ArrayList currList = (ArrayList)eventsHashTable[gBatteryCapacityChanged];
                currList.Remove(value);
            }
        }

        /// <summary>
        /// Gets all power schemes
        /// </summary>
        /// <returns>Dictionary having key is power scheme guid and value is its friendly name</returns>
        private static Dictionary<string, Guid> GetPowerSchemes()
        {
            Guid SchemeGuid = Guid.Empty;
            uint index = 0;
            uint BufferSize = (uint)Marshal.SizeOf(SchemeGuid);

            Dictionary<string, Guid> schemes = new Dictionary<string, Guid>();
            while (0 == PowerEnumerate((IntPtr)null, (IntPtr)null, (IntPtr)null, POWER_DATA_ACCESSOR.ACCESS_SCHEME, index, ref SchemeGuid, ref BufferSize))
            {
                schemes.Add(GetPowerSchemeName(SchemeGuid), SchemeGuid);

                index++;
            }

            return schemes;
        }

        /// <summary>
        /// Gets friendly name for the power scheme from guid
        /// </summary>
        /// <param name="SchemeGuid">Guid of power scheme of which name is required.</param>
        /// <returns>Friendly name of power scheme from SchemeGuid</returns>
        private static string GetPowerSchemeName(Guid SchemeGuid)
        {
            string name = string.Empty;
            IntPtr lpszName = (IntPtr)null;
            uint dwSize = 0;
            PowerReadFriendlyName((IntPtr)null, ref SchemeGuid, (IntPtr)null, (IntPtr)null, lpszName, ref dwSize);
            if (dwSize > 0)
            {
                lpszName = Marshal.AllocHGlobal((int)dwSize);
                if (0 == PowerReadFriendlyName((IntPtr)null, ref SchemeGuid, (IntPtr)null, (IntPtr)null, lpszName, ref dwSize))
                {
                    name = Marshal.PtrToStringUni(lpszName);
                }
                if (lpszName != IntPtr.Zero)
                    Marshal.FreeHGlobal(lpszName);
            }

            return name;
        }

        /// <summary>
        /// Gets Guid of currently active power scheme.
        /// </summary>
        /// <returns>Guid of active power scheme</returns>
        public static Guid GetActivePowerScheme()
        {
            Guid activeScheme = Guid.Empty;

            IntPtr ptr = Marshal.AllocHGlobal(Marshal.SizeOf(typeof(IntPtr)));
            if (0 == PowerGetActiveScheme((IntPtr)null, out ptr))
            {
                activeScheme = (Guid)Marshal.PtrToStructure(ptr, typeof(Guid));

                if (ptr != null)
                {
                    Marshal.FreeHGlobal(ptr);
                }
            }

            return activeScheme;
        }

        /// <summary>
        /// Activates power scheme provided by gPowerScheme.
        /// </summary>
        /// <param name="gPowerScheme">Guid of power scheme to active</param>
        public static void SetActivePowerScheme(Guid gPowerScheme)
        {
            PowerSetActiveScheme((IntPtr)null, ref gPowerScheme);
        }

        /// <summary>
        /// Activates the "Balanced" power scheme.
        /// </summary>
        public static void ActiveBalancedPowerScheme()
        {
            Dictionary<string, Guid> schemes = GetPowerSchemes();
            foreach (var sc in schemes)
            {
                if (sc.Key.ToLower().Contains("balance"))
                {
                    SetActivePowerScheme(sc.Value);
                    break;
                }
            }
        }

        /// <summary>
        /// Activates the "Power Saver" power scheme.
        /// </summary>
        public static void ActivePowerSaverScheme()
        {
            Dictionary<string, Guid> schemes = GetPowerSchemes();
            foreach (var sc in schemes)
            {
                if (sc.Key.ToLower().Contains("saver"))
                {
                    SetActivePowerScheme(sc.Value);
                    break;
                }
            }
        }

        public PowerMngrHelper(Window handlerWindow)
        {
            interopHelper = new WindowInteropHelper(handlerWindow);
        }

        /// <summary>
        /// Fires any registered event handlers.
        /// </summary>
        /// <param name="eventsHandlersList">ArrayList of events handlers.</param>
        private static void FireEvents(ArrayList eventsHandlersList)
        {
            foreach (EventHandler handler in eventsHandlersList)
            {
                handler.Invoke(null, new EventArgs());
            }
        }

        // Used to catch the first message which Windows sends when register for a power notification.
        // We dont need to fire any event handlers when this occurs.
        private bool powerSrcChangeHandled;
        private bool batteryLifeChangeHandled;
        
        /// <summary>
        /// Checks whether the message has been handled or not
        /// This is used when an event is registered first time.
        /// </summary>
        /// <param name="gEvent">The event to check.</param>
        /// <returns>A boolean value. Returns true if the 
        /// message should be caught.</returns>
        private bool IsMsgHandled(Guid gEvent)
        {
            bool ifMsgCaught = false;

            if (gEvent == gBatteryCapacityChanged)
            {
                if (!batteryLifeChangeHandled)
                {
                    batteryLifeChangeHandled = true;
                    ifMsgCaught = true;
                }
            }
            else if (gEvent == gPowerSourceChanged)
            {
                if (!powerSrcChangeHandled)
                {
                    powerSrcChangeHandled = true;
                    ifMsgCaught = true;
                }
            }

            return ifMsgCaught;
        }

        /// <summary>
        /// This method is called when a Windows message 
        /// is sent to this window.
        /// The method calls the registered event handlers.
        /// </summary>
        //protected override void WndProc(ref Message m)
        IntPtr WindowProc(IntPtr hwnd, int msg, IntPtr wParam, IntPtr lParam, ref bool handled)
        {
            // Make sure it is a Power Management message.
            if (msg == PowerBroadcastMsg &&
                (int)wParam == PowerSettingChangeMsg)
            {
                PowerSetting ps =
                     (PowerSetting)Marshal.PtrToStructure(
                         lParam, typeof(PowerSetting));

                IntPtr pData = new IntPtr(lParam.ToInt64() + Marshal.SizeOf(ps));
                Guid currentEvent = ps.PowerSettingChanged;

                if (!IsMsgHandled(currentEvent))
                {
                    FireEvents((ArrayList)eventsHashTable[currentEvent]);
                }

                handled = true;
            }

            return IntPtr.Zero;
        }
    }
}
