﻿/*

Copyright (c) <2012>, Intel Corporation All Rights Reserved.
 
The source code, information and material ("Material") contained herein is owned by Intel Corporation or its suppliers or licensors, 
and title to such Material remains with Intel Corporation or its suppliers or licensors. The Material contains proprietary information 
of Intel or its suppliers and licensors. The Material is protected by worldwide copyright laws and treaty provisions. No part of the Material 
may be used, copied, reproduced, modified, published, uploaded, posted, transmitted, distributed or disclosed in any way without Intel's 
prior express written permission. No license under any patent, copyright or other intellectual property rights in the Material is granted 
to or conferred upon you, either expressly, by implication, inducement, estoppel or otherwise. Any license under such intellectual 
property rights must be express and approved by Intel in writing.
 
Unless otherwise agreed by Intel in writing, you may not remove or alter this notice or any other notice embedded in Materials by Intel 
or Intel’s suppliers or licensors in any way.

*/

using System;
using System.Collections.ObjectModel;
using PhotoApp.DataServices;
using Windows.Devices.Sensors;
using PhotoApp.Utility;
using System.ServiceModel;
using System.IO;
using System.Drawing.Imaging;
using System.Drawing;
using System.Text;
using System.Linq;
using System.Windows;
using Logger;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace PhotoApp.ViewModel
{
    /// <remarks>
    /// ViewModel of MainWindow. Provides mechanism to communicate with Model and View components of the system.
    /// Derives property changed notification from ViewModelBase class.
    /// </remarks>
    class MainWindowViewModel : ViewModelBase
    {
        //Minimum battery threshold value.
        private const double BatteryThreshold = 25.0;

        //Member variables
        private ObservableCollection<string> _imagesList;
        private string _largeImagePath;
        private string _powerState;
        private string _orientationMode;
        private string _ambientLightValue;
        private string _brightnessValue;
        private string _geoLatitude;
        private string _geoLongitude;
        //private string _civicAddress;
        private bool _geoLocationEnable;
        private string _imageTitle;
        private Visibility _imageGeoWebIconVisibile;
        private Visibility _locationDbgPnlGrdVisibility;
        private ImageSource _powerImageSource;
        private SimpleOrientationSensor orientationSensor;
        private DisplayBrightnessHelper brightnessHelper;
        private LightSensor lightSensor;
        private Accelerometer accelerometer;
        private GeoLocationHelper geoLocator;
        private Guid originalPowerScheme;
        private double currentLatitude;
        private double currentLongitude;
        //private string currentCivicAddress;
        private bool powerSaverActivated;
        public bool userClickedGeoTagBtn;
        //Current orientation.
        public ViewOrientationMode viewCurrentOrientation;

        /// <summary>
        /// Current light sesnor reading.
        /// </summary>
        private float LightSensorReading
        {
            set
            {
                AmbientLightValue = string.Format("{0} Lux", value);
            }
        }
        /**************************************************************************************************************        
                Supported Properties
        ***************************************************************************************************************/
        /// <summary>
        /// List of image paths.
        /// </summary>
        public ObservableCollection<string> ImageList
        {
            get
            {
                return _imagesList;
            }
            private set
            {
                if (_imagesList != value)
                {
                    _imagesList = value;
                    OnNotifyPropertyChanged("ImageList");
                }
            }
        }

        /// <summary>
        /// Large image path which is displaed in large central view.
        /// </summary>
        public string LargeImagePath
        {
            get
            {
                return _largeImagePath;
            }
            private set
            {
                if (_largeImagePath != value)
                {
                    _largeImagePath = value;
                    OnNotifyPropertyChanged("LargeImagePath");
                }
            }
        }

        /// <summary>
        /// Current battery state in percentage.
        /// </summary>        
        public string PowerState
        {
            get
            {
                return _powerState;
            }
            private set
            {
                if (_powerState != value)
                {
                    _powerState = value;
                    OnNotifyPropertyChanged("PowerState");
                }
            }
        }

        /// <summary>
        /// Current orientation of the device.
        /// </summary>        
        public string OrientationMode
        {
            get
            {
                return _orientationMode;
            }
            private set
            {
                if (_orientationMode != value)
                {
                    _orientationMode = value;
                    OnNotifyPropertyChanged("OrientationMode");
                }
            }
        }

        /// <summary>
        /// Current Ambient light.
        /// </summary>       
        public string AmbientLightValue
        {
            get
            {
                return _ambientLightValue;
            }
            private set
            {
                if (_ambientLightValue != value)
                {
                    _ambientLightValue = value;
                    OnNotifyPropertyChanged("AmbientLightValue");
                }
            }
        }

        /// <summary>
        /// Current display brightness value in percentage.
        /// </summary>
        public string BrightnessValue
        {
            get
            {
                return _brightnessValue;
            }
            private set
            {
                if (_brightnessValue != value)
                {
                    _brightnessValue = value;
                    OnNotifyPropertyChanged("BrightnessValue");
                }
            }
        }

        /// <summary>
        /// Displays geo position latitude value
        /// </summary>
        public string GeoLatitude
        {
            get
            {
                return _geoLatitude;
            }
            private set
            {
                if (_geoLatitude != value)
                {
                    _geoLatitude = value;
                    OnNotifyPropertyChanged("GeoLatitude");
                }
            }
        }

        /// <summary>
        /// Displays geo position longitude value
        /// </summary>
        public string GeoLongitude
        {
            get
            {
                return _geoLongitude;
            }
            private set
            {
                if (_geoLongitude != value)
                {
                    _geoLongitude = value;
                    OnNotifyPropertyChanged("GeoLongitude");
                }
            }
        }

        /// <summary>
        /// Displays geo civic address
        /// </summary>
        /*public string CivicAddress
        {
            get
            {
                return _civicAddress;
            }
            private set
            {
                if (_civicAddress != value)
                {
                    _civicAddress = value;
                    OnNotifyPropertyChanged("CivicAddress");
                }
            }
        }*/

        /// <summary>
        /// Enables or disables geo location
        /// </summary>
        public bool GeoLocationEnable
        {
            get
            {
                return _geoLocationEnable;
            }
            set
            {
                if (_geoLocationEnable != value)
                {
                    _geoLocationEnable = value;
                    OnNotifyPropertyChanged("GeoLocationEnable");
                }

                if (value == true)
                    StartGeoLoctor();
                else
                    StopGeoLocator();
            }
        }


        /// <summary>
        /// Hides or Shows geo location information in debug panel.
        /// </summary>
        public Visibility LocationDbgPnlGrdVisibility
        {
            get
            {
                return _locationDbgPnlGrdVisibility;
            }
            set
            {
                if (_locationDbgPnlGrdVisibility != value)
                {
                    _locationDbgPnlGrdVisibility = value;
                    OnNotifyPropertyChanged("LocationDbgPnlGrdVisibility");
                }
            }
        }

        /// <summary>
        /// Sets or Gets image source for power mode
        /// </summary>
        public ImageSource PowerImageSource
        {
            get
            {
                return _powerImageSource;
            }
            set
            {
                if (_powerImageSource != value)
                {
                    _powerImageSource = value;
                    OnNotifyPropertyChanged("PowerImageSource");
                }
            }
        }

        /// <summary>
        /// Image title, bind to title of image.
        /// </summary>
        public string ImageTitle
        {
            get
            {
                return _imageTitle;
            }
            set
            {
                if (_imageTitle != value)
                {
                    _imageTitle = value;
                    OnNotifyPropertyChanged("ImageTitle");
                }
            }
        }

        /// <summary>
        /// Shows or hides web icon in image title.
        /// </summary>
        private Visibility ImageGeoWebIconVisibile
        {
            get
            {
                return _imageGeoWebIconVisibile;
            }
            set
            {
                //if (_imageGeoWebIconVisibile != value)
                {
                    _imageGeoWebIconVisibile = value;
                    OnNotifyPropertyChanged("ImageGeoWebIconVisibile");
                }
            }
        }

        /// <summary>
        /// Event handler to notify the orientation changed event.
        /// </summary>
        public event EventHandler<OrientationChangedEventArgs> OrientationChanged;

        /// <summary>
        /// Event handler to notify the device has been shake.
        /// </summary>
        public event EventHandler<EventArgs> DeviceShaked;

        /// <summary>
        /// Event handler to notify the battery percentage is below threshold value.
        /// </summary>
        public event EventHandler<EventArgs> BatteryBelowThreshold;

        /// <summary>
        /// Event handler to notify the battery percentage is above threshold value.
        /// </summary>
        public event EventHandler<EventArgs> BatteryAboveThreshold;

        /**************************************************************************************************************        
                Supported Operations: Supports different operations to be performed. Provides ready-to-use
                data to view and also accesses model. This mainly uses Power Management Utility.
        ***************************************************************************************************************/
        /// <summary>
        /// Detects current power source using PowerMngrHelper and assigns value to "PowerState" property.
        /// </summary>
        public void SetPowerSource()
        {
            //SYSTEM_BATTERY_STATE batt_status = PowerMngrHelper.GetBatteryState();
            SYSTEM_POWER_STATUS power_status = PowerMngrHelper.GetPowerState();

            if (power_status.LineStatus == ACLineStatus.Online) // AC Power
            {
                PowerState = "AC Power";  //System is running on AC power.

                //Restore to previous power scheme.
                if (originalPowerScheme != null)
                    PowerMngrHelper.SetActivePowerScheme(originalPowerScheme);

                //Power saver scheme is not currently active.
                powerSaverActivated = false;

                //Write log about power source changed.
                AppLogger.LogToFile("Current power source is AC power");

                //Set the power mode image to show charging.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath + "charging.png", UriKind.RelativeOrAbsolute));
                if (bi != null) PowerImageSource = bi;

                //Notify the view about battery above threshold value(i.e. power plugged in), so that view can enable the camera/location feature.
                if (BatteryAboveThreshold != null)
                    BatteryAboveThreshold(this, new EventArgs());
            }
            else
            {
                //Write log about power source changed.
                AppLogger.LogToFile("Current power source is Battery power");

                //Store the current power scheme.
                originalPowerScheme = PowerMngrHelper.GetActivePowerScheme();

                SetBatteryLife();   //System is running on battery power so show the battery remain percentage.

                //Set the power mode image to show battery.
                BitmapImage bi = new BitmapImage(new Uri(AppConstants.ImagesPath + "battery.png", UriKind.RelativeOrAbsolute));
                if (bi != null) PowerImageSource = bi;
            }
        }
        /// <summary>
        /// Detects current battery level using PowerMngrHelper and assigns value to "PowerState" property.
        /// </summary>
        public void SetBatteryLife()
        {
            //Set the battery percentage.
            SYSTEM_POWER_STATUS power_status = PowerMngrHelper.GetPowerState();
            PowerState = string.Format("{0}%", power_status.BatteryLifePercent.ToString());

            //Check whether battery under threshold value or not.
            if (power_status.BatteryLifePercent < BatteryThreshold && power_status.LineStatus == ACLineStatus.Offline)
            {
                //Notify the view about battery below threshold value, so that view can disable the camera/location feature.
                if (BatteryBelowThreshold != null)
                    BatteryBelowThreshold(this, new EventArgs());

                //Activate the "Power Saver" power scheme.
                if (!powerSaverActivated)
                {
                    PowerMngrHelper.ActivePowerSaverScheme();
                    //Power saver scheme is currently active.
                    powerSaverActivated = true;
                }
            }
            else
            {
                //Notify the view about battery above threshold value, so that view can enable the camera/location feature.
                if (BatteryAboveThreshold != null)
                    BatteryAboveThreshold(this, new EventArgs());

                //Battery is now above threshold, so activate the saved power scheme, only if current active scheme is power saver.
                if (powerSaverActivated)
                {
                    //Restore to previous power scheme.
                    if (originalPowerScheme != null)
                        PowerMngrHelper.SetActivePowerScheme(originalPowerScheme);
                    powerSaverActivated = false;
                }
            }

            //Write log about power source changed.
            AppLogger.LogToFile("Current Battery status is " + PowerState);
        }

        /// <summary>
        /// Checks battery is below threshold or not.
        /// </summary>
        /// <returns>Returns true if battery percentage below threhold value, else returns false</returns>
        public bool IsBatteryBelowThreshold()
        {
            SYSTEM_POWER_STATUS powerState = PowerMngrHelper.GetPowerState();
            if (powerState.LineStatus == ACLineStatus.Offline && powerState.BatteryLifePercent <= BatteryThreshold)
                return true;

                return false;
        }

        /// <summary>
        /// Does few initialization stuff.
        /// Creates instance of LightSensor and registers for notification events.
        /// Creates instance of SimpleOrientationSensor and registers for notification events.
        /// Detects current power state and sets value to properties.
        /// </summary>
        public MainWindowViewModel()
        {
            //Ambient Light sensor.
            lightSensor = LightSensor.GetDefault();
            if (lightSensor != null)    //If it is null then device doesn't have light sensor.
            {
                //Register for the event to get the sensor value changed notification.
                lightSensor.ReadingChanged += lightSensor_ReadingChanged;

                //Get the current reading.
                LightSensorReading = lightSensor.GetCurrentReading().IlluminanceInLux;
            }

            //Orientation Sensor.
            orientationSensor = SimpleOrientationSensor.GetDefault();
            if (orientationSensor != null)  //If it is null then device doesn't have orientation sensor.
            {
                //Register for the orientation changed notification.
                orientationSensor.OrientationChanged += orientationSensor_OrientationChanged;

                //Set the orientation mode initially.
                SetOrientationModeProperty(orientationSensor.GetCurrentOrientation());
            }

            //Register for the brightness changed notification.
            brightnessHelper = new DisplayBrightnessHelper();
            brightnessHelper.BrightnessChanged += brightnessHelper_BrightnessChanged;
            //Set the current brightness value initially.
            SetBrightnessProperty();

            //Accelerometer Sensor
            accelerometer = Accelerometer.GetDefault();
            if (accelerometer != null)  //If it is null then device doesn't have accelerometer sensor.
            {
                //Register for device shaken notification.
                accelerometer.Shaken += accelerometer_Shaken;
            }

            //GeoLocator to get the geo location.
            geoLocator = new GeoLocationHelper();
            if (geoLocator != null)
            {
                geoLocator.positionChanged += geoLocator_positionChanged;
            }

            //Set the power source value initially.
            SetPowerSource();
        }

        //Event invoked when geo postion changed.
        void geoLocator_positionChanged(object sender, GeoPositionChangedEventArgs e)
        {
            currentLatitude = e.geoData.latitude;
            currentLongitude = e.geoData.longitude;

            //Set the latitude - longitude value.
            GeoLatitude = string.Format("Latitude: {0}", currentLatitude.ToString());
            GeoLongitude = string.Format("Longitude: {0}", currentLongitude.ToString());

            //Write log about geo position changed.
            string LatLong = string.Format("Lat: {0} Lon: {1}", e.geoData.latitude, e.geoData.longitude);
            AppLogger.LogToFile("Current geo poistion" + LatLong);

            
        }

        //Starts the GeoLocator to get the geo position.
        void StartGeoLoctor()
        {
            if (geoLocator != null)
                geoLocator.StartGettingLocationUpdate();

            //Show location information in debug panel.
            LocationDbgPnlGrdVisibility = Visibility.Visible;
        }

        //Stops the GeoLocator.
        void StopGeoLocator()
        {
            if (geoLocator != null)
                geoLocator.StopGettingLocationUpdate();

            //Reset the geo strings.
            GeoLatitude = GeoLongitude = string.Empty;
            //CivicAddress = string.Empty;

            //Hide location information in debug panel.
            LocationDbgPnlGrdVisibility = Visibility.Collapsed;
        }

        /// <summary>
        /// Notification method called when device shakes
        /// </summary>
        /// <param name="sender">Sender</param>
        /// <param name="args">Event arguments</param>
        void accelerometer_Shaken(Accelerometer sender, AccelerometerShakenEventArgs args)
        {
            //Notify view about device shaked.
            if (DeviceShaked != null)
                DeviceShaked(this, new EventArgs());
        }

        /// <summary>
        /// Notification method called when brightness value changes.
        /// </summary>
        /// <param name="sender">Sender</param>
        /// <param name="e">EventArgs</param>
        void brightnessHelper_BrightnessChanged(object sender, EventArgs e)
        {
            //Brightness value changed so update the property accordingly.
            SetBrightnessProperty();
        }

        /// <summary>
        /// Sets the brightness value.
        /// </summary>
        void SetBrightnessProperty()
        {
            byte br = DisplayBrightnessHelper.GetCurrentBrightness();

            int brightness = Convert.ToInt16(br);

            if (brightness > 0)
                BrightnessValue = string.Format("{0}%", brightness);
            else
                BrightnessValue = "";

            //Write log about brightness value changed.
            AppLogger.LogToFile("Current brightness value is " + BrightnessValue);

        }

        /// <summary>
        /// Should be called by Window while closing to unregister all the event handlers.
        /// </summary>
        public void UnregisterHandlers()
        {
            //Unregister the brightness changed event handler.
            brightnessHelper.BrightnessChanged -= brightnessHelper_BrightnessChanged;

            //Restore to original power scheme.
            if (originalPowerScheme != null)
                PowerMngrHelper.SetActivePowerScheme(originalPowerScheme);

            //Stop geo locator.
            StopGeoLocator();
        }

        /// <summary>
        /// Sets current orienttaion value to "OrientationMode" property.
        /// </summary>
        /// <param name="orientation"> current orientation</param>
        void SetOrientationModeProperty(SimpleOrientation orientation)
        {
            ViewOrientationMode vOrientation = ViewOrientationMode.OrientationModeUnknown;

            switch (orientation)
            {
                case SimpleOrientation.NotRotated:
                    {
                        OrientationMode = "Portrait";
                        vOrientation = ViewOrientationMode.OrientationModePortraitUp;
                        break;
                    }
                case SimpleOrientation.Rotated90DegreesCounterclockwise:
                    {
                        OrientationMode = "Landscape";
                        vOrientation = ViewOrientationMode.OrientationModeLandScapeLeft;
                        break;
                    }
                case SimpleOrientation.Rotated180DegreesCounterclockwise:
                    {
                        OrientationMode = "Portrait";
                        vOrientation = ViewOrientationMode.OrientationModePortraitDown;
                        break;
                    }
                case SimpleOrientation.Rotated270DegreesCounterclockwise:
                    {
                        OrientationMode = "Landscape";
                        vOrientation = ViewOrientationMode.OrientationModeLandScapeRight;
                        break;
                    }
                case SimpleOrientation.Faceup:
                    {
                        OrientationMode = "Faceup";
                        vOrientation = ViewOrientationMode.OrientationModeFaceUp;
                        break;
                    }
                case SimpleOrientation.Facedown:
                    {
                        OrientationMode = "Facedown";
                        vOrientation = ViewOrientationMode.OrientationModeFaceDown;
                        break;
                    }
                default:
                    {
                        OrientationMode = "Unknown";
                        break;
                    }
            }

            viewCurrentOrientation = vOrientation;

            //Write log about orientation changed.
            AppLogger.LogToFile("Orientation changed to " + OrientationMode);

            //Show the toast message about orientation changed.
            //ToastMsgHelper.ShowToastMessage("Orientation changed to " + OrientationMode);

            if (OrientationChanged != null)
                OrientationChanged(this, new OrientationChangedEventArgs(vOrientation));
        }

        /// <summary>
        /// Event method called when device's orientation is changed.
        /// </summary>
        /// <param name="sender">Represents orientation sensor.</param>
        /// <param name="args">Sensor orienttaion details.</param>
        void orientationSensor_OrientationChanged(SimpleOrientationSensor sender, SimpleOrientationSensorOrientationChangedEventArgs args)
        {
            SetOrientationModeProperty(args.Orientation);
        }

        /// <summary>
        /// Event method called when ambient light is changed.
        /// </summary>
        /// <param name="sender">Represents light sensor.</param>
        /// <param name="args">Light sensor details.</param>
        void lightSensor_ReadingChanged(LightSensor sender, LightSensorReadingChangedEventArgs args)
        {
            LightSensorReading = args.Reading.IlluminanceInLux;

            //Write log about light sensor reading changed.
            AppLogger.LogToFile("Current ambient light value is " + AmbientLightValue);
        }

        /// <summary>
        /// Load all images from "pictures" directory to images array.
        /// </summary>
        public void LoadImages()
        {
            DataService ds = new DataService();
            string[] imagesPath = ds.GetPathsOfAllStoredImages();

            if (imagesPath != null)
                ImageList = new ObservableCollection<string>(imagesPath);
        }


        /// <summary>
        /// Shows larger image for the specified index in images array.
        /// </summary>
        /// <param name="index">Index of image in array</param>
        public void ShowLargeImage(int index)
        {
            ImageTitle = "";
            ImageGeoWebIconVisibile = Visibility.Collapsed;

            if (index >= 0 && index < ImageList.Count)
            {
                LargeImagePath = ImageList[index];

                //Set the image title.
                Image im = Image.FromFile(LargeImagePath);
                if (im != null)
                {
                    PropertyItem[] pp = im.PropertyItems;

                    if (pp != null)
                    {
                        //Find the property item for title.
                        var selection = from p in pp
                                        where p.Id == 0x010E
                                        select p;

                        if (selection != null && selection.Count() > 0)
                        {
                            ASCIIEncoding encoding = new ASCIIEncoding();
                            string imgTitle = encoding.GetString(selection.ElementAt(0).Value);

                            //Remove null character.
                            ImageTitle = imgTitle.Substring(0, imgTitle.Length - 1);

                            //Show the geo icon.
                            ImageGeoWebIconVisibile = Visibility.Visible;
                        }
                    }
                }
            }
        }

    }
}
